use futures::stream::StreamExt;
use std::{
    collections::HashMap,
    fmt::Display,
    time::{SystemTime, UNIX_EPOCH},
};
use upower_dbus::{BatteryLevel, DeviceProxy, UPowerProxy};
use zbus::zvariant::OwnedObjectPath;

enum ChargeState {
    Charging,
    Discharging,
}

enum EnergyRate {
    Watts(f64),
}

impl From<f64> for EnergyRate {
    fn from(value: f64) -> Self {
        Self::Watts(value)
    }
}

enum BatteryCharge {
    Voltage(f64),
}

impl From<f64> for BatteryCharge {
    fn from(value: f64) -> Self {
        Self::Voltage(value)
    }
}

struct BatteryStatus {
    charge: BatteryCharge,
    charge_percentage: f64,
    charge_state: ChargeState,
    rate: EnergyRate,
}

impl<'a> From<DeviceProxy<'a>> for BatteryStatus {
    fn from(value: DeviceProxy) -> Self {
        todo!()
    }
}

type BatteryIdentifier = String;

struct Entry {
    timestamp: SystemTime,
    batteries: HashMap<BatteryIdentifier, BatteryStatus>,
}

fn main() -> zbus::Result<()> {
    futures::executor::block_on(async move {
        let connection = zbus::Connection::system().await?;

        let upower = UPowerProxy::new(&connection).await?;
        let device_enumerator = upower.enumerate_devices().await?;
        for d in device_enumerator {
            eprintln!("Device: {d}");
        }

        let device = upower.get_display_device().await?;
        eprintln!("Display Device: {device:?}");
        let batteries = HashMap::new();
        let does_contain = batteries.contains_key(&device.path().to_string());

        println!("Battery Percentage: {:?}", device.percentage().await);

        let tracker = Entry {
            timestamp: SystemTime::now(),
            batteries,
        };

        println!("On Battery: {:?}", upower.on_battery().await);

        let mut stream = upower.receive_on_battery_changed().await;

        while let Some(event) = stream.next().await {
            println!("On Battery Changed: {:?}", event.get().await);
        }

        Ok(())
    })
}
